/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is ForumZilla 0.1.1.
 *
 * The Initial Developer of the Original Code is Zapogee, Inc.
 * Portions created by Zapogee, Inc. are Copyright (C) 2001
 * Zapogee, Inc.  All Rights Reserved.
 *
 * Contributor(s):
 *   Myk Melez, <myk@melez.com>, original author
 */

function FeedService() {}


FeedService.prototype.initFeedPolling = function() {
    // Flag to indicate whether or not we're in the process of storing an item.
    // Used by storeItem() to determine when to start storing the next item.
    this.itemBeingStored = false;

    FeedService.periodicallyDownloadFeeds();

    window.setTimeout(FeedService.periodicallyFlushDataSources, 5 * 60 * 1000);

    //window.setTimeout(FeedService.periodicallyStoreItems, 5 * 1000);
    window.setTimeout(FeedService.periodicallyStoreItem, 100);
}
    
FeedService.prototype.__defineGetter__('pollingInterval', function() {
    return nsPreferences.getIntPref("forumzilla.polling.interval", DEFAULT_POLLING_INTERVAL);
});

FeedService.prototype.__defineGetter__('storageMode', function() {
    return nsPreferences.getIntPref("forumzilla.storage.mode", DEFAULT_STORAGE_MODE);
});

FeedService.prototype.__defineGetter__('storageFolder', function() {
    return nsPreferences.copyUnicharPref("forumzilla.storage.folder", DEFAULT_STORAGE_FOLDER);
});

// Download a feed.
FeedService.prototype.downloadFeed = function(feed) {
    var ds = getSubscriptionsDS();
    new Feed(feed).download();
}

// Download feeds and check for new messages.
FeedService.prototype.downloadFeeds = function() {
    debug("polling feeds");

    var ds = getSubscriptionsDS();
    var feeds = getSubscriptionsList().GetElements();
  
    var now = new Date().getTime();
    while(feeds.hasMoreElements()) {
        var feed = new Feed(feeds.getNext());
        var then = feed.lastPolled ? feed.lastPolled.getTime() : 0;
        var elapsed= (now - then)/1000/60;
        if (elapsed >= feed.interval)
            feed.download();
        else
            debug(feed.url + " not downloaded; " + elapsed.toPrecision(2) + "/" + feed.interval);
    }
}


// Items to store (i.e. items that have been downloaded
// but haven't been stored into a mail folder yet).  Processed by storeItems().
//
FeedService.itemsToStore = new Array();

// NOTE: storeItems() isn't used at the moment because of a bug in the copy
// service that causes it not to call the copy listener's OnStopCopy() method
// sometimes when copying multiple messages to an IMAP folder.  Instead, we
// copy messages one at a time by setting a flag to indicate when a message
// is being copied and calling storeItem() repeatedly to copy the next message
// if the flag indicates the previous copy has completed.

/*
// Stores the items in the queue.
FeedService.prototype.storeItems = function() {
    item = FeedService.itemsToStore.shift();
    while (item) {
        item.store();
        item = FeedService.itemsToStore.shift();
    }
}

// Periodically calls storeItems() to store the items in the queue.
FeedService.periodicallyStoreItems = function() {
    window.setTimeout(FeedService.periodicallyStoreItems, 5 * 1000);
    feedService.storeItems();
}
*/


// Stores the next item in the queue.
FeedService.prototype.storeItem = function() {
    if (!this.itemBeingStored) {
        this.itemBeingStored = true;
        item = FeedService.itemsToStore.shift();
        if (item)
            item.store();
        else
            this.itemBeingStored = false;
    }
}

// Periodically calls storeItem() to store the next item in the queue.
FeedService.periodicallyStoreItem = function() {
    window.setTimeout(FeedService.periodicallyStoreItem, 100);
    feedService.storeItem();
}


// Feeds to parse.
FeedService.feedsToParse = new Object();

// Periodically calls downloadFeeds() to download feeds and check for new messages.
FeedService.periodicallyDownloadFeeds = function() {
    // Periodically download and process feeds.
    window.setTimeout(FeedService.periodicallyDownloadFeeds, 60 * 1000);
    feedService.downloadFeeds();
}

FeedService.prototype.flushDataSources = function() {
    var ds;

    try {
        ds = getSubscriptionsDS();
        ds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
        if (ds)
            ds.Flush();
    }
    catch(e) {
        debug("couldn't flush subscriptions data source: " + e);
    }

    try {
        ds = getItemsDS();
        ds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
        if (ds)
            ds.Flush();
    }
    catch(e) {
        debug("couldn't flush items data source: " + e);
    }
}

// Periodically flushes data sources to prevent an application crash
// from losing changes.
FeedService.periodicallyFlushDataSources = function() {
    // Periodically download and process feeds.
    window.setTimeout(FeedService.periodicallyFlushDataSources, 5 * 60 * 1000);
    feedService.flushDataSources();
}
