/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is ForumZilla 0.1.1.
 *
 * The Initial Developer of the Original Code is Zapogee, Inc.
 * Portions created by Zapogee, Inc. are Copyright (C) 2001
 * Zapogee, Inc.  All Rights Reserved.
 *
 * Contributor(s):
 *   Myk Melez, <myk@melez.com>, original author
 */

/*
 * Global Variables
 */

var gCurrentSelection = -1;
var gSomethingChanged = 0;

var feedService = new FeedService();


/*
 * Event Handlers
 */

function doLoad() {
    // Display the list of feed subscriptions.
    var ds = getSubscriptionsDS();
    var tree = document.getElementById('subscriptions');
    tree.database.AddDataSource(ds);
    tree.builder.rebuild();
    if (tree.view.rowCount > 0)
        tree.view.selection.select(0);
    tree.focus();
}

function doClose() {
    // Redownload all feeds.  We should only download the last modified one,
    // since the others have all already been downloaded (if changed by the user
    // and then unselected), but that doesn't seem to work since we have to refer
    // to the feed somehow, and the reference gets destroyed when this window
    // is closed.
    window.opener.setTimeout("feedService.downloadFeeds();", 1000);
    window.close();
}

function doAdd() {
    var tree = document.getElementById('subscriptions');
    var url = window.prompt("Location:", "");
    if (url == null)
        return;
    add(url);

}
function add(url, title) {
    var tree = document.getElementById('subscriptions');
    var ds = getSubscriptionsDS();
    var feeds = getSubscriptionsList();

    // Generate a unique ID for the feed.
    var id = url;
    var i = 1;
    while (feeds.IndexOf(rdf.GetResource(id)) != -1 && ++i < 1000)
        id = url + i;
    if (id == 1000)
        throw("couldn't generate a unique ID for feed " + url);
    id = rdf.GetResource(id);

    // Add the feed to the subscriptions list.
    feeds.AppendElement(id);

    // Store information about the feed.
    ds.Assert(id, RDF_TYPE, FZ_FEED, true);
    ds.Assert(id, DC_IDENTIFIER, rdf.GetLiteral(url), true);
    ds.Assert(id, FZ_QUICKMODE, rdf.GetLiteral("false"), true);
    ds.Assert(id, FZ_INTERVAL, rdf.GetLiteral(feedService.pollingInterval), true);

    // Create a new feed object for the feed.
    feed = new Feed(id);

    if (title) {
        ds.Assert(id, DC_TITLE, rdf.GetLiteral(title), true);
    }
    else {
        // Downloading the feed synchronously will pick up the title.
        feed.download(false, false);
    }

    if (feedService.storageMode == FOLDER_STORAGE_MODE)
        ds.Assert(id, FZ_FOLDER, rdf.GetResource(feedService.storageFolder), true);
    else if (feedService.storageMode == SUBFOLDERS_STORAGE_MODE) {
        var folder = GetMsgFolderFromUri(feedService.storageFolder, true);
        folder.createSubfolder(feed.title, window.opener.msgWindow);
        //var subfolder = folder.FindSubFolder(feed.title).URI;
        var subfolder = feedService.storageFolder + "/" + encodeURIComponent(feed.title);
        ds.Assert(id, FZ_FOLDER, rdf.GetResource(subfolder), true);
    }

    tree.builder.rebuild();
    tree.view.selection.select(tree.view.rowCount-1);
    document.getElementById('title').focus();

    gSomethingChanged = 1;
}

function doImport() {
    var rv = pickOpen('Pick OPML File to Import', '$xml $all');
    if (rv.reason == PICK_CANCEL)
        return false;

    var file = new LocalFile(rv.file, MODE_RDONLY);
    var text = file.read();
    var gParser = new DOMParser;
    var xmlDocument = gParser.parseFromUTF8String(text, 'text/xml');
    var feeds = xmlDocument.getElementsByTagName("body")[0].getElementsByTagName("outline");
    for ( var i=0 ; i<feeds.length ; i++ ) {
        var feed = feeds[i];
        debug("importing from OPML feed " + feed.getAttribute("title"));
        if (feed.getAttribute("type") == "rss" && feed.getAttribute("title"))
            add(feed.getAttribute("xmlUrl"), feed.getAttribute("title"));
    }
    return true;
}

function doRemove() {
    var tree = document.getElementById('subscriptions');
    var selectedIndex = tree.view.selection.currentIndex;
    var treeitem = tree.view.getItemAtIndex(selectedIndex);
    var feed = rdf.GetResource(treeitem.id);

    // Remove the feed from the list of subscriptions.
    var feeds = getSubscriptionsList();
    var index = feeds.IndexOf(feed);
    if (index != -1)
        feeds.RemoveElementAt(index, false);

    // Remove all assertions about the feed from the subscriptions database.
    var ds = getSubscriptionsDS();
    removeAssertions(ds, feed);

    // Remove all assertions about items in the feed from the items database.
    ds = getItemsDS();
    var items = ds.GetSources(FZ_FEED, feed, true);
    while (items.hasMoreElements()) {
        var item = items.getNext();
        item = item.QueryInterface(Components.interfaces.nsIRDFResource);
        ds.Unassert(item, FZ_FEED, feed, true);
        if (ds.hasArcOut(item, FZ_FEED))
            debug(item.Value + " is from more than one feed; only the reference to this feed removed");
        else
            removeAssertions(ds, item);
    }

    tree.builder.rebuild();
    if (tree.view.rowCount > selectedIndex)
        tree.view.selection.select(selectedIndex);
    else if (tree.view.rowCount > 0)
        tree.view.selection.select(tree.view.rowCount-1);
    tree.focus();
}

function doCheck() {
    var tree = document.getElementById('subscriptions');
    downloadFeedAtIndex(tree.view.selection.currentIndex);
}

function doSelect(event) {
    var tree = document.getElementById('subscriptions');
    var titleField = document.getElementById('title');
    var urlField = document.getElementById('url');
    var folderField = document.getElementById('actionTargetFolder');
    var intervalField = document.getElementById('interval');
    var editCaption = document.getElementById('edit-caption');

    if (!gSomethingChanged) debug('nothing changed');
    if (gSomethingChanged && gCurrentSelection != -1 && gCurrentSelection != tree.view.selection.currentIndex)
        downloadFeedAtIndex(gCurrentSelection);
    gCurrentSelection = tree.view.selection.currentIndex;
    gSomethingChanged = 0;

    if (tree.view.selection.currentIndex == -1) {
        titleField.value = "";
        urlField.value = "";
        SetFolderPicker(feedService.storageFolder, 'actionTargetFolder');
        intervalField.value = "";
        editCaption.setAttribute('label', "Edit Subscription");

        //titleField.setAttribute('disabled', true);
        //urlField.setAttribute('disabled', true);
        //folderField.setAttribute('disabled', true);
        //intervalField.setAttribute('disabled', true);
    }
    else {
        var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
        var resource = rdf.GetResource(item.id);
        var ds = getSubscriptionsDS();
    
        var feeds = getSubscriptionsList();
        var index = feeds.IndexOf(resource);
    
        var title = ds.GetTarget(resource, DC_TITLE, true);
        title = title ? title.QueryInterface(Components.interfaces.nsIRDFLiteral).Value : "";
        var url = ds.GetTarget(resource, DC_IDENTIFIER, true);
        url = url ? url.QueryInterface(Components.interfaces.nsIRDFLiteral).Value : "";
        var folder = ds.GetTarget(resource, FZ_FOLDER, true);
        folder = folder ? folder.QueryInterface(Components.interfaces.nsIRDFResource).Value : feedService.storageFolder;
        var interval = ds.GetTarget(resource, FZ_INTERVAL, true);
        interval = interval ? interval.QueryInterface(Components.interfaces.nsIRDFLiteral).Value : "";
    
        //titleField.setAttribute('disabled', false);
        //urlField.setAttribute('disabled', false);
        //folderField.setAttribute('disabled', false);
        //intervalField.setAttribute('disabled', false);

        titleField.value = title;
        urlField.value = url;
        SetFolderPicker(folder, 'actionTargetFolder');
        intervalField.value = interval;
        editCaption.setAttribute('label', "Edit Feed: " + title);
    }
}

function doUpdateTitle() {
    var tree = document.getElementById('subscriptions');
    var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
    var ds = getSubscriptionsDS();
    var resource = rdf.GetResource(item.id);
    var new_title = document.getElementById('title').value;
    var old_title = ds.GetTarget(resource, DC_TITLE, true);

    if (old_title)
        ds.Change(resource, DC_TITLE, old_title, rdf.GetLiteral(new_title));
    else
        ds.Assert(resource, DC_TITLE, rdf.GetLiteral(new_title), true);

    document.getElementById('edit-caption').setAttribute('label', "Edit Feed: " + new_title);
}

function doUpdateURL() {
    var tree = document.getElementById('subscriptions');
    var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
    var ds = getSubscriptionsDS();
    var resource = rdf.GetResource(item.id);
    var new_url = document.getElementById('url').value;
    var old_url = ds.GetTarget(resource, DC_IDENTIFIER, true);

    if (old_url)
        ds.Change(resource, DC_IDENTIFIER, old_url, rdf.GetLiteral(new_url));
    else
        ds.Assert(resource, DC_IDENTIFIER, rdf.GetLiteral(new_url), true);

    gSomethingChanged = 1;
}

function doUpdateInterval() {
    var tree = document.getElementById('subscriptions');
    var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
    var ds = getSubscriptionsDS();
    var resource = rdf.GetResource(item.id);
    var new_interval = document.getElementById('interval').value;
    var old_interval = ds.GetTarget(resource, FZ_INTERVAL, true);

    if (old_interval)
        ds.Change(resource, FZ_INTERVAL, old_interval, rdf.GetLiteral(new_interval));
    else
        ds.Assert(resource, FZ_INTERVAL, rdf.GetLiteral(new_interval), true);
}

function doUpdateFolder() {
    var tree = document.getElementById('subscriptions');
    var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
    var ds = getSubscriptionsDS();
    var resource = rdf.GetResource(item.id);
    var new_folder = document.getElementById('actionTargetFolder').getAttribute('uri');
    var old_folder = ds.GetTarget(resource, FZ_FOLDER, true);

    if (old_folder)
        ds.Change(resource, FZ_FOLDER, old_folder, rdf.GetResource(new_folder));
    else
        ds.Assert(resource, FZ_FOLDER, rdf.GetResource(new_folder), true);

    var index = tree.view.selection.currentIndex;
    tree.builder.rebuild();
    tree.view.selection.select(index);

    gSomethingChanged = 1;
}


function toggleMode() {
    var modeButton = document.getElementById('mode-button');
    var advancedSection = document.getElementById('advanced-section');
    if (modeButton.label == "Advanced...") {
        advancedSection.style.display = "block";
        modeButton.label = "Simple...";
    }
    else {
        modeButton.label = "Advanced...";
        advancedSection.style.display = "none";
    }
}

/*
 * Stuff
 */

function downloadFeedAtIndex(index) {
    var tree = document.getElementById('subscriptions');
    var ds = getSubscriptionsDS();

    var item;
    try {
        item = tree.view.getItemAtIndex(index);
    } catch(e) {
        debug("couldn't get unselected item at index " + index);
        return;
    }
    var feed = rdf.GetResource(item.id);
    window.opener.feedService.downloadFeed(feed);
    //window.opener.feedService.storeItems();
}
